'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

require('babel-polyfill');
var test = _interopDefault(require('ava'));
var reduxSaga = require('redux-saga');
var reduxSaga_effects = require('redux-saga/effects');
var reduxSaga_utils = require('redux-saga/utils');
require('ramda');
var build_TEMP = require('./');
var erlangJs = require('erlang-js');

/**
 * Testing utils
 *
 * @namespace TestUtils
 *
 */

var resumeIteratorAndCheckOutput = function resumeIteratorAndCheckOutput(t, iterator) {
  return function (into, outof) {
    var value = void 0;
    if (!iterator) {
      throw 'iterator is undefined';
    }

    try {
      value = iterator.next(into).value;
      t.deepEqual(value, outof);
    } catch (e) {
      t.deepEqual(e.message, outof.error.message);
    }
  };
};

/**
 * checks generator scenario
 *
 * this is equivalent to:
 * asserting that two iterators are equal, when given the same input
 *
 * @param {NodeTap} t node-tap instance
 * @param {Iterator} iter
 * @param {[]} scenario Scenario data
 * @returns {void}
 *
 * @memberof TestUtils
 */
var checkGeneratorScenario = function checkGeneratorScenario(t, iter, scenario) {
  var next = resumeIteratorAndCheckOutput(t, iter);

  for (var i = 0; i < scenario.length - 1; i += 2) {
    var input = scenario[i];
    var output = scenario[i + 1];
    next(input, output);
  }
};

var _marked = [worker, workerOne, workerTwo].map(regeneratorRuntime.mark);

/* eslint-disable import/no-unresolved */
// Data protocol examples

function worker() {
  return regeneratorRuntime.wrap(function worker$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return reduxSaga_effects.call(reduxSaga.delay, 1000);

        case 2:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked[0], this);
}
function workerOne() {
  return regeneratorRuntime.wrap(function workerOne$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return reduxSaga_effects.call(reduxSaga.delay, 1000);

        case 2:
        case 'end':
          return _context2.stop();
      }
    }
  }, _marked[1], this);
}
function workerTwo() {
  return regeneratorRuntime.wrap(function workerTwo$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          _context3.next = 2;
          return reduxSaga_effects.call(reduxSaga.delay, 1000);

        case 2:
        case 'end':
          return _context3.stop();
      }
    }
  }, _marked[2], this);
}

var mockWorker = reduxSaga_utils.createMockTask();
var mockWorkerOne = reduxSaga_utils.createMockTask();
var mockWorkerTwo = reduxSaga_utils.createMockTask();
var permanentWorkerSpec = { start: worker, restart: build_TEMP.PERMANENT, id: '0' };
var temporaryWorkerSpec = { start: worker, restart: build_TEMP.TEMPORARY, id: '0' };
var firstWorkerSpec = { start: workerOne, restart: build_TEMP.PERMANENT, id: 'first' };
var secondWorkerSpec = { start: workerTwo, restart: build_TEMP.PERMANENT, id: 'second' };
var workerRace = reduxSaga_effects.race({ '0': reduxSaga_effects.join(mockWorker) });
var workersRace = reduxSaga_effects.race({ 'first': reduxSaga_effects.join(mockWorkerOne), 'second': reduxSaga_effects.join(mockWorkerTwo) });
var workerTerminated = { '0': '0' };
var firstWorkerTerminated = { 'first': 'first' };
var secondWorkerTerminated = { 'second': 'second' };
var waitedLongEnough = { waitedLongEnough: true };
var restartFreqError = { error: { message: 'Restart frequency limit reached' } };
var timedWorkerRace = reduxSaga_effects.race({ waitedLongEnough: reduxSaga_effects.call(reduxSaga.delay, 1000, true), '0': reduxSaga_effects.join(mockWorker) });
var timedWorkersRace = reduxSaga_effects.race({
  waitedLongEnough: reduxSaga_effects.call(reduxSaga.delay, 1000, true),
  'first': reduxSaga_effects.join(mockWorkerOne),
  'second': reduxSaga_effects.join(mockWorkerTwo)
});

var sys = void 0;
test.beforeEach(function (t) {
  sys = new erlangJs.ProcessSystem();
});

test('erl system works', function (t) {
  var pid = sys.spawn(regeneratorRuntime.mark(function _callee() {
    return regeneratorRuntime.wrap(function _callee$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return 1;

          case 2:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee, this);
  }));

  t.is(sys.list().length, 2);
  t.is(sys.list()[1], pid);
});

test('supervisor: one-for-one, max restart frequency', function (t) {
  // initialization
  var supervisorIter = build_TEMP.supervisor.init({
    supFlags: { strategy: build_TEMP.ONE_FOR_ONE, maxR: 1, maxT: 1 },
    childSpecs: [permanentWorkerSpec]
  });

  checkGeneratorScenario(t, supervisorIter, [null, reduxSaga_effects.fork(worker), mockWorker, // spawns first (and only) worker

  // main loop:

  reduxSaga_effects.race({ // yields worker and delay race
    0: reduxSaga_effects.join(mockWorker),
    waitedLongEnough: reduxSaga_effects.call(reduxSaga.delay, 1, true)
  }), waitedLongEnough, // if waited long enough,
  workerRace, // workers race is yielded

  workerTerminated, // when worker terminates,
  reduxSaga_effects.fork(worker), mockWorker, // it gets restarted


  reduxSaga_effects.race({ // yields worker and delay race
    0: reduxSaga_effects.join(mockWorker),
    waitedLongEnough: reduxSaga_effects.call(reduxSaga.delay, 1, true)
  }), workerTerminated, // if worker terminates more frequently than permitted,
  restartFreqError // supervisor dies

  ]);
});

test('supervisor: one-for-one, multi', function (t) {
  // initialization
  var supervisorIter = build_TEMP.supervisor.init({
    supFlags: { strategy: build_TEMP.ONE_FOR_ONE },
    childSpecs: [firstWorkerSpec, secondWorkerSpec]
  });

  checkGeneratorScenario(t, supervisorIter, [null, reduxSaga_effects.fork(workerOne), mockWorkerOne, // spawns first worker
  reduxSaga_effects.fork(workerTwo), mockWorkerTwo, // spawns second worker

  // main loop:

  timedWorkersRace, // yields workers and delay race
  waitedLongEnough, // if delay does trigger,
  workersRace, // yields workers race

  firstWorkerTerminated, // when first worker terminates,
  reduxSaga_effects.fork(workerOne), mockWorkerOne, // it gets restarted

  timedWorkersRace, // the same with worker 1
  waitedLongEnough, workersRace, secondWorkerTerminated, reduxSaga_effects.fork(workerTwo), mockWorkerTwo, timedWorkersRace, firstWorkerTerminated, // if any worker terminates,
  restartFreqError // supervisor dies
  ]);
});

test('supervisor: all-for-one, multi', function (t) {
  // initialization
  var supervisorIter = build_TEMP.supervisor.init({
    supFlags: { strategy: build_TEMP.ONE_FOR_MANY },
    childSpecs: [firstWorkerSpec, secondWorkerSpec]
  });

  checkGeneratorScenario(t, supervisorIter, [null, reduxSaga_effects.fork(workerOne), mockWorkerOne, // initialization:
  reduxSaga_effects.fork(workerTwo), mockWorkerTwo, // spawns workers

  // main loop:
  timedWorkersRace, // listens for workers termination
  waitedLongEnough, workersRace, firstWorkerTerminated, // when first worker terminates:
  reduxSaga_effects.fork(workerOne), mockWorkerOne, // it gets restarted,

  reduxSaga_effects.cancel(mockWorkerTwo), null, // and second gets restarted too
  reduxSaga_effects.fork(workerTwo), mockWorkerTwo]);
});
//# sourceMappingURL=test.js.map
